/*
 * dbtool - a simple but powerfull commandline interface
 *      to the GNU gdbm system, usefull for shellscripts
 *      which needs a database for data storage.
 *
 * Copyright (c) 2001 Thomas Linden, <tom@daemon.de>
 * All Rights Reserved.      Released under the GPL.
 *
 * $Id: dbtool.h,v 1.2 2001/04/23 21:11:39 scip Exp $
 */


/* the default initial blocksize for the gdbm database, see gdbm(3) */
#define BLOCKSIZE 1024

/* the default filemode for file creation, will be xored to the users umask */
#define FILEMODE  0644

/* the name of the environment variable to be used if -d is omited */
#define DB "DB_DATABASE"

/* normally VERSION should be specified by configure */
#ifndef VERSION
#define VERSION 1.2
#endif

/* normally PACKAGE should be specified by configure */
#ifndef PACKAGE
#define PACKAGE db
#endif

#include <string>


#ifdef HAVE_BERKELEY

#ifndef NO_SYSTEM_INCLUDES
#include <sys/types.h>
#include <stdlib.h>
#include <string.h>
#ifndef _MSC_VER
#include <unistd.h>
#endif
#endif

#include <db_cxx.h>

#else

# include <gdbm.h>

#endif

/*
 * The Config class will hold all flags, modes and other
 * parameters we need for running.
 * methods defined in cmd.cc
 */
class Config {
 private:
  int     _argc;
  char  **_argv;
  string pkg;
  char *default_db;

 public:
  Config(int argc, char *argv[]);
  int parse();
  int     force, command, with, reverse;
  string  filename, key, value, usage, token;
  char    separator;
};



/*
 * The Engine class will do all gdbm related stuff, i.e.
 * insert, search, create, remove and so on.
 * methods defined in engine.cc
 */
class Engine {
 private:
  void init(const Config& config); /* init() will be called from every member once */

#ifdef HAVE_BERKELEY
  Db *db;
#else
  GDBM_FILE db;
#endif

  string pkg;

 public:
  Engine() {;;};
  void insert(const Config& config);
  void update(const Config& config);
  void remove(const Config& config);
  void select(const Config& config);
  void  stdin(const Config& config);
  void   dump(const Config& config);
  void regexp(const Config& config);
};
